from dataclasses import is_dataclass

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators import random_data_class_generator
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomDictGenerator(RandomGenerator):
    """
    Class to generate random dict
    """

    def generate(self, *args, **kwargs) -> dict:
        max_length = kwargs.get(defaults.MAX_LENGTH_ARG, defaults.MAX_DICT_LENGTH)

        default_key_type = kwargs.get(
            defaults.DEFAULT_KEY_TYPE_ARG, defaults.DEFAULT_KEY_TYPE
        )
        default_value_type = kwargs.get(
            defaults.DEFAULT_VALUE_TYPE_ARG, defaults.DEFAULT_VALUE_TYPE
        )

        key_type = kwargs.get(defaults.KEY_TYPE_ARG, default_key_type)
        value_type = kwargs.get(defaults.VALUE_TYPE_ARG, default_value_type)

        if is_dataclass(key_type):
            key_generator = random_data_class_generator.RandomDataClassGenerator()
        else:
            key_generator = defaults.TYPING_GENERATORS[key_type]()

        if is_dataclass(value_type):
            value_generator = random_data_class_generator.RandomDataClassGenerator()
        else:
            value_generator = defaults.TYPING_GENERATORS[value_type]()

        random_dict = {}
        for _ in range(max_length):
            if is_dataclass(key_type):
                dict_key = key_generator.generate(key_type)
            else:
                dict_key = key_generator.generate()

            if is_dataclass(value_type):
                dict_value = value_generator.generate(value_type)
            else:
                dict_value = value_generator.generate()

            random_dict[dict_key] = dict_value

        return random_dict



import random

from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomBoolGenerator(RandomGenerator):
    """
    Class to generate random booleans
    """

    def generate(self, *args, **kwargs) -> bool:
        random_bit = random.getrandbits(1)
        return bool(random_bit)



from pathlib import Path

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator
from dataclass_bakery.generators.random_str_generator import RandomStrGenerator


class RandomPathGenerator(RandomGenerator):
    def generate(self, *args, **kwargs) -> Path:
        generator = RandomStrGenerator()
        max_path_length = kwargs.get(defaults.MAX_LENGTH_ARG, defaults.MAX_PATH_LENGTH)
        folders = [generator.generate() for _ in range(max_path_length)]
        random_path = "/".join(folders)
        return Path(random_path)



from decimal import Decimal
from pathlib import Path
from typing import Literal
from uuid import UUID

from dataclass_bakery.generators.random_bool_generator import RandomBoolGenerator
from dataclass_bakery.generators.random_complex_generator import RandomComplexGenerator
from dataclass_bakery.generators.random_decimal_generator import RandomDecimalGenerator
from dataclass_bakery.generators.random_dict_generator import RandomDictGenerator
from dataclass_bakery.generators.random_float_generator import RandomFloatGenerator
from dataclass_bakery.generators.random_int_generator import RandomIntGenerator
from dataclass_bakery.generators.random_list_generator import RandomListGenerator
from dataclass_bakery.generators.random_option_generator import RandomOptionGenerator
from dataclass_bakery.generators.random_path_generator import RandomPathGenerator
from dataclass_bakery.generators.random_range_generator import RandomRangeGenerator
from dataclass_bakery.generators.random_set_generator import RandomSetGenerator
from dataclass_bakery.generators.random_str_generator import RandomStrGenerator
from dataclass_bakery.generators.random_tuple_generator import RandomTupleGenerator
from dataclass_bakery.generators.random_uuid_generator import RandomUuidGenerator

# Generators
TYPING_GENERATORS = {
    str: RandomStrGenerator,
    int: RandomIntGenerator,
    float: RandomFloatGenerator,
    complex: RandomComplexGenerator,
    bool: RandomBoolGenerator,
    dict: RandomDictGenerator,
    list: RandomListGenerator,
    tuple: RandomTupleGenerator,
    set: RandomSetGenerator,
    range: RandomRangeGenerator,
    Decimal: RandomDecimalGenerator,
    Path: RandomPathGenerator,
    UUID: RandomUuidGenerator,
    Literal: RandomOptionGenerator,
}

# Generator Default Values
NUMBER_MIN_LIMIT = 0
NUMBER_MAX_LIMIT = 100

DECIMALS = 2

DEFAULT_KEY_TYPE = str
DEFAULT_VALUE_TYPE = int
DEFAULT_PATH_TYPE = str

MAX_STR_LENGTH = 10
MAX_LIST_LENGTH = 2
MAX_TUPLE_LENGTH = 2
MAX_SET_LENGTH = 2
MAX_DICT_LENGTH = 2
MAX_PATH_LENGTH = 2

# Bakery Arguments
FIXED_VALUE_ARG = "_fixed_value_"  # A value to directly assig to the attr
GENERATOR_ARG = "_generator_"  # Generator to change the default generator
KEY_TYPE_ARG = "_key_type_"  # Key type in a dict
VALUE_TYPE_ARG = "_value_type_"  # Value type in a list, tuple or dict
OPTIONS_ARG = "_options_"  # Options in RandomOptionGenerator
NUMBER_MIN_LIMIT_ARG = "_min_limit_"  # Fix a min limit of a number
NUMBER_MAX_LIMIT_ARG = "_max_limit_"  # Fix a max limit of a number
DECIMALS_ARG = "_decimals_"  # Fix how many decimals has a number
MAX_LENGTH_ARG = "_max_length_"  # Fix length in a attr
DEFAULT_KEY_TYPE_ARG = "_default_key_type_"  # Fix the default key typing
DEFAULT_VALUE_TYPE_ARG = "_default_value_type_"  # Fix the default value typing



from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomSetGenerator(RandomGenerator):
    """
    Class to generate random set
    """

    def generate(self, *args, **kwargs) -> set:
        max_length = kwargs.get(defaults.MAX_LENGTH_ARG, defaults.MAX_SET_LENGTH)

        default_value_type = kwargs.get(
            defaults.DEFAULT_VALUE_TYPE_ARG, defaults.DEFAULT_VALUE_TYPE
        )

        value_type = kwargs.get(defaults.VALUE_TYPE_ARG, default_value_type)
        generator = defaults.TYPING_GENERATORS[value_type]()

        random_set = set()
        for _ in range(max_length):
            set_value = generator.generate()
            random_set.add(set_value)

        return random_set



import random
from typing import Any

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomOptionGenerator(RandomGenerator):
    """
    Class to generate random option
    """

    def generate(self, *args, **kwargs) -> Any:
        START_POSITION = 1
        options = kwargs[defaults.OPTIONS_ARG]
        positions = len(options) - 1
        random_position = random.randint(START_POSITION, positions)
        return options[random_position]



from typing import Any, Dict, List, Optional, Union

from dataclass_bakery.generators.random_data_class_generator import (
    RandomDataClassGenerator,
)


def make(
    _data_class: Any,
    _quantity: Optional[int] = 1,
    _attr_defaults: Optional[Dict] = {},
) -> Union[List, Any]:
    random_data_class_generator = RandomDataClassGenerator()

    data_class_objects = []
    for _ in range(_quantity):
        data_class_object = random_data_class_generator.generate(
            _data_class, **_attr_defaults
        )
        data_class_objects.append(data_class_object)

    if _quantity > 1:
        return data_class_objects

    return data_class_object



import random

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomIntGenerator(RandomGenerator):
    """
    Class to generate random int number
    """

    def generate(self, *args, **kwargs) -> int:
        min_limit = kwargs.get(defaults.NUMBER_MIN_LIMIT_ARG, defaults.NUMBER_MIN_LIMIT)
        max_limit = kwargs.get(defaults.NUMBER_MAX_LIMIT_ARG, defaults.NUMBER_MAX_LIMIT)
        random_number = random.randint(min_limit, max_limit)
        return random_number



from uuid import UUID, uuid4

from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomUuidGenerator(RandomGenerator):
    """
    Class to generate uuid
    """

    def generate(self, *args, **kwargs) -> UUID:
        return uuid4()



from dataclasses import is_dataclass
from typing import _GenericAlias, Any, Literal, Union, Dict

from dataclass_bakery.generators import defaults


def collect_all_attributes(dclass: Any) -> Dict:
    if not is_dataclass(dclass):
        return {}
    attrs = dict(dclass.__annotations__.items())
    for pclass in dclass.__bases__:
        attrs.update(collect_all_attributes(pclass))

    return attrs


class RandomDataClassGenerator:
    def generate(self, data_class: Any, *args, **kwargs) -> Any:
        random_data = {}
        for field_name, field_type in collect_all_attributes(data_class).items():
            arguments = kwargs.get(field_name, {})
            if defaults.FIXED_VALUE_ARG in arguments:  # Value fixed
                random_data[field_name] = arguments[defaults.FIXED_VALUE_ARG]
                continue

            if defaults.GENERATOR_ARG in arguments:  # Generator fixed
                generator = arguments[defaults.GENERATOR_ARG]()
                field_randomized = generator.generate(**arguments)
                random_data[field_name] = field_randomized
                continue

            if isinstance(field_type, _GenericAlias) and field_type.__origin__ == Union:
                has_new_type = False
                for argument in field_type.__args__:
                    if argument != type(None):
                        has_new_type = True
                        field_type = argument

                    if not hasattr(field_type, "__origin__") or field_type.__origin__ != Union:
                        break

                if not has_new_type:
                    raise TypeError(f"Union without Typing in dataclass {field_name}")

            if (
                isinstance(field_type, _GenericAlias) and field_type.__origin__ != Literal
            ):  # Is a Dict or List
                arguments_lenght = len(field_type.__args__)
                if arguments_lenght > 1:  # Is a Dict
                    arguments[defaults.KEY_TYPE_ARG] = field_type.__args__[0]
                    arguments[defaults.VALUE_TYPE_ARG] = field_type.__args__[1]
                elif arguments_lenght > 0:  # Is a List or Tuple
                    arguments[defaults.VALUE_TYPE_ARG] = field_type.__args__[0]

                field_type = field_type.__origin__

            elif isinstance(field_type, _GenericAlias) and field_type.__origin__ == Literal:
                arguments[defaults.OPTIONS_ARG] = field_type.__args__
                field_type = field_type.__origin__

            if is_dataclass(field_type):
                generator = RandomDataClassGenerator()
                field_randomized = generator.generate(field_type, **arguments)
            else:
                generator_class = defaults.TYPING_GENERATORS.get(field_type)
                generator = generator_class()
                field_randomized = generator.generate(**arguments)

            random_data[field_name] = field_randomized

        return data_class(**random_data)



import random

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomFloatGenerator(RandomGenerator):
    """
    Class to generate random float number
    """

    def generate(self, *args, **kwargs) -> float:
        min_limit = kwargs.get(defaults.NUMBER_MIN_LIMIT_ARG, defaults.NUMBER_MIN_LIMIT)
        max_limit = kwargs.get(defaults.NUMBER_MAX_LIMIT_ARG, defaults.NUMBER_MAX_LIMIT)
        decimals = kwargs.get(defaults.DECIMALS_ARG, defaults.DECIMALS)
        random_number = random.uniform(min_limit, max_limit)
        rounded_number = round(random_number, decimals)
        return rounded_number



import random

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomComplexGenerator(RandomGenerator):
    """
    Class to generate random complex number
    """

    def generate(self, *args, **kwargs) -> complex:
        min_limit = kwargs.get(defaults.NUMBER_MIN_LIMIT_ARG, defaults.NUMBER_MIN_LIMIT)
        max_limit = kwargs.get(defaults.NUMBER_MAX_LIMIT_ARG, defaults.NUMBER_MAX_LIMIT)

        decimals = kwargs.get(defaults.DECIMALS_ARG, defaults.DECIMALS)
        real_part = random.uniform(min_limit, max_limit)
        rounded_real_part = round(real_part, decimals)
        imaginary_part = random.uniform(min_limit, max_limit)
        rounded_imaginary_part = round(imaginary_part, decimals)
        return complex(rounded_real_part, rounded_imaginary_part)



from dataclasses import is_dataclass

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators import random_data_class_generator
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomTupleGenerator(RandomGenerator):
    """
    Class to generate random tuple
    """

    def generate(self, *args, **kwargs) -> tuple:
        max_length = kwargs.get(defaults.MAX_LENGTH_ARG, defaults.MAX_TUPLE_LENGTH)

        default_value_type = kwargs.get(
            defaults.DEFAULT_VALUE_TYPE_ARG, defaults.DEFAULT_VALUE_TYPE
        )

        value_type = kwargs.get(defaults.VALUE_TYPE_ARG, default_value_type)

        if is_dataclass(value_type):
            generator = random_data_class_generator.RandomDataClassGenerator()
        else:
            generator = defaults.TYPING_GENERATORS[value_type]()

        random_tuple = []
        for _ in range(max_length):
            if is_dataclass(value_type):
                tuple_value = generator.generate(value_type)
            else:
                tuple_value = generator.generate()
            random_tuple.append(tuple_value)

        return tuple(random_tuple)



import random, string

from dataclass_bakery.generators import defaults
from dataclass_bakery.generators.random_generator import RandomGenerator


class RandomStrGenerator(RandomGenerator):
    """
    Class to generate random string
    """

    def generate(self, *args, **kwargs) -> str:
        max_length = kwargs.get(defaults.MAX_LENGTH_ARG, defaults.MAX_STR_LENGTH)
        characters = string.ascii_uppercase + string.ascii_lowercase
        random_choices = random.choices(characters, k=max_length)
        random_str = "".join(random_choices)
        return random_str
